@@ -2,7 +2,7 @@
  * conf.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -92,7 +92,7 @@ void Conf::save_list() {
   save_list("locations", locations);
 }
 
-void Conf::save_conf(char *_file, std::map<std::string, std::string>& mp) {
+void Conf::save_conf(const char *_file, std::map<std::string, std::string>& mp) {
   if (conf_dir.length()) {
     std::string file = conf_dir;
     file += _file;
@@ -108,7 +108,7 @@ void Conf::save_conf(char *_file, std::map<std::string, std::string>& mp) {
   }
 }
 
-void Conf::save_list(char * _file, std::vector<std::string>& mp) {
+void Conf::save_list(const char * _file, std::vector<std::string>& mp) {
   if (conf_dir.length()) {
     std::string file = conf_dir;
     file += _file;
@@ -124,14 +124,7 @@ void Conf::save_list(char * _file, std::vector<std::string>& mp) {
 }
 
 bool Conf::prepare_dir() {
-  conf_dir = Glib::build_filename(Glib::get_home_dir(),
-#ifdef _WIN32
-				  "Katoob"
-#else
-				  ".katoob"
-#endif
-				  );
-  conf_dir += Utils::get_dir_separator();
+  conf_dir = (Utils::get_conf_dir() + Utils::get_dir_separator());
 
   // We check that it's there and is a directory.
   if (!Glib::file_test (conf_dir, Glib::FILE_TEST_IS_DIR)) {
@@ -151,19 +144,26 @@ bool Conf::prepare_dir() {
       return false;
     }
   }
+
+  // Now our crashed documents directory.
+  if (!Glib::file_test (Utils::get_recovery_dir(), Glib::FILE_TEST_IS_DIR)) {
+    if (g_mkdir (Utils::get_recovery_dir().c_str(), 0700)) {
+      g_warning ("Can't create recovery directory");
+      return false;
+    }
+  }
   return true;
 }
 
 void Conf::load_conf() {
-  load_conf("config", config);
+  _ok = load_conf("config", config);
   load_conf("print", print);
   adjust_lists();
 }
 
-void Conf::load_conf(char *_file, std::map<std::string, std::string>& mp) {
+bool Conf::load_conf(const char *_file, std::map<std::string, std::string>& mp) {
   if (conf_dir.length()) {
-    std::string file = conf_dir;
-    file += _file;
+    std::string file(conf_dir + _file);
     std::ifstream ifs;
     ifs.open(file.c_str());
     if (ifs.is_open()) {
@@ -182,9 +182,10 @@ void Conf::load_conf(char *_file, std::map<std::string, std::string>& mp) {
 	mp[key] = value;
       }
       ifs.close();
-      _ok = true;
+      return true;
     }
   }
+  return false;
 }
 
 void Conf::load_list() {
@@ -193,7 +194,7 @@ void Conf::load_list() {
   load_list("locations", locations);
 }
 
-void Conf::load_list(gchar *_file, std::vector<std::string>& mp) {
+void Conf::load_list(const gchar *_file, std::vector<std::string>& mp) {
   if (conf_dir.length()) {
     std::string file = conf_dir;
     file += _file;
@@ -209,7 +210,23 @@ void Conf::load_list(gchar *_file, std::vector<std::string>& mp) {
   }
 }
 
-int Conf::_get(std::map<std::string, std::string>& m, char *key, int val) {
+double Conf::_get(std::map<std::string, std::string>& m, const char *key, double val) {
+  std::map<std::string, std::string>::iterator iter;
+  iter = m.find(key);
+  if (iter == m.end()) {
+    // We'll set it.
+    std::stringstream str;
+    str << val;
+    m[key] = str.str();
+#ifndef NDEBUG
+    std::cerr << __PRETTY_FUNCTION__ << " Shouldn't be here, Got key " << key << std::endl;
+#endif
+    return val;
+  }
+  return atof(iter->second.c_str());
+}
+
+int Conf::_get(std::map<std::string, std::string>& m, const char *key, int val) {
   std::map<std::string, std::string>::iterator iter;
   iter = m.find(key);
   if (iter == m.end()) {
@@ -225,7 +242,7 @@ int Conf::_get(std::map<std::string, std::string>& m, char *key, int val) {
   return atoi(iter->second.c_str());
 }
 
-bool Conf::_get(std::map<std::string, std::string>& m, char *key, bool val) {
+bool Conf::_get(std::map<std::string, std::string>& m, const char *key, bool val) {
   std::map<std::string, std::string>::iterator iter;
   iter = m.find(key);
   if (iter == m.end()) {
@@ -241,7 +258,7 @@ bool Conf::_get(std::map<std::string, std::string>& m, char *key, bool val) {
   return atoi(iter->second.c_str()) == 0 ? false : true;
 }
 
-std::string Conf::_get(std::map<std::string, std::string>& m, char *key, std::string& val) {
+std::string Conf::_get(std::map<std::string, std::string>& m, const char *key, std::string& val) {
   std::map<std::string, std::string>::iterator iter;
   iter = m.find(key);
   if (iter == m.end()) {
@@ -255,7 +272,13 @@ std::string Conf::_get(std::map<std::string, std::string>& m, char *key, std::st
   return iter->second;
 }
 
-void Conf::_set(std::map<std::string, std::string>& m, char *key, int val) {
+void Conf::_set(std::map<std::string, std::string>&m, const char *key, double val) {
+  std::stringstream str;
+  str << val;
+  m[key] = str.str();
+}
+
+void Conf::_set(std::map<std::string, std::string>& m, const char *key, int val) {
   std::stringstream str;
   str << val;
   m[key] = str.str();
