@@ -2,7 +2,7 @@
  * dbus.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -96,10 +96,19 @@ DBusHandlerResult DBus::got_message(DBusConnection *connection, DBusMessage *mes
   else if (method == OPEN_FILES) {
     return open_files(connection, message);
   }
+#ifdef ENABLE_MAEMO
+  else if (method == "top_application") {
+    signal_request_top.emit();
+    //    DBusMessage *reply = dbus_message_new_method_return(message);
+    //    dbus_connection_send(connection, reply, NULL);
+    //    dbus_message_unref(reply);
+    return DBUS_HANDLER_RESULT_HANDLED;
+  }
+#endif
   else {
+    std::cerr << "Katoob: Unhandled DBus event: " << method << std::endl;
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
   }
-  //  std::cout << dbus_message_get_member (message) << std::endl;
 }
 
 DBusHandlerResult DBus::open_files(DBusConnection *connection, DBusMessage *message) {
@@ -156,7 +165,7 @@ bool DBus::ping() {
   DBusMessage *msg, *res;
   DBusError error;
   dbus_error_init(&error);
-  char *_ping = PING;
+  const char *_ping = PING;
 
   con = dbus_bus_get(DBUS_BUS_SESSION, &error);
   if (!con) {
