@@ -2,7 +2,7 @@
  * dialogs.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@
 #include <gtkmm/label.h>
 #include <gtkmm/spinbutton.h>
 #include <gtkmm/adjustment.h>
+#include <gtkmm/scrolledwindow.h>
+#include <gtkmm/textview.h>
 #include "dialogs.hh"
 #include "macros.h"
 
@@ -53,12 +55,38 @@ void katoob_info(const std::string& message) {
   dlg.run();
 }
 
+bool katoob_big_info(const std::string& message, bool enable_insert) {
+  Gtk::Dialog dlg;
+  Gtk::ScrolledWindow sw;
+  sw.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
+  Gtk::TextView tv;
+  tv.set_editable(false);
+  tv.get_buffer()->set_text(message);
+  sw.add(tv);
+  dlg.get_vbox()->pack_start(sw, true, true);
+  dlg.set_size_request(Gdk::Screen::get_default()->get_width()/2, Gdk::Screen::get_default()->get_height()/2);
+
+  if (enable_insert) {
+    dlg.add_button(_("Insert to document"), Gtk::RESPONSE_YES);
+  }
+
+  dlg.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
+
+  dlg.show_all();
+  switch (dlg.run()) {
+  case Gtk::RESPONSE_YES:
+    return true;
+  default:
+    return false;
+  }
+}
+
 int katoob_question(const std::string& message) {
   Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
 
-  dialog.add_button(Gtk::Stock::YES, Gtk::RESPONSE_YES);
-  dialog.add_button(Gtk::Stock::NO, Gtk::RESPONSE_NO);
   dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  dialog.add_button(Gtk::Stock::NO, Gtk::RESPONSE_NO);
+  dialog.add_button(Gtk::Stock::YES, Gtk::RESPONSE_YES);
   dialog.set_position (Gtk::WIN_POS_CENTER);
 
   return dialog.run();
@@ -103,3 +131,32 @@ int katoob_goto_dialog() {
     return -1;
   }
 }
+
+Glib::RefPtr<ActivityMeter> katoob_activity() {
+  return Glib::RefPtr<ActivityMeter>(new ActivityMeter);
+}
+
+ActivityMeter::ActivityMeter() : dialog(_("Please wait..."), true, true) {
+  loop = Glib::MainLoop::create();
+  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  dialog.get_vbox()->pack_start(bar, true, true);
+  bar.set_pulse_step(0.007);
+  dialog.signal_response().connect(sigc::mem_fun(this, &ActivityMeter::signal_response_cb));
+}
+
+void ActivityMeter::signal_response_cb(int res) {
+  quit();
+  signal_cancel_clicked.emit();
+}
+
+void ActivityMeter::run() {
+  conn = Glib::signal_idle().connect(sigc::bind_return(sigc::mem_fun(bar, &Gtk::ProgressBar::pulse), true), Glib::PRIORITY_LOW);
+  dialog.show_all();
+  loop->run();
+}
+
+void ActivityMeter::quit() {
+  conn.disconnect();
+  dialog.hide();
+  loop->quit();
+}
