@@ -2,7 +2,7 @@
  * document.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
+#include <cerrno>
 #include <string>
 #include <iostream>
 #include <sstream>
@@ -39,6 +40,9 @@
 #include "macros.h"
 #include "textbuffer.hh"
 #include "utils.hh"
+#ifdef ENABLE_MAEMO
+#include <gtkmm/settings.h>
+#endif
 
 // TODO:
 // highlight current line
@@ -65,6 +69,12 @@ void _on_toggle_overwrite(GtkTextView *textview, gpointer user_data) {
   static_cast<Document *>(user_data)->on_toggle_overwrite();
 }
 
+#ifdef ENABLE_MAEMO
+void __tap_and_hold(GtkWidget *widget, gpointer user_data) {
+  static_cast<Document *>(user_data)->_tap_and_hold();
+}
+#endif
+
 Document::Document(Conf& conf, Encodings& encodings, int num) :
   _label(conf),
   _conf(conf),
@@ -76,14 +86,11 @@ Document::Document(Conf& conf, Encodings& encodings, int num) :
   _line_numbers(false),
   __on_move_cursor(0),
   __on_toggle_overwrite(0),
-  _overwrite(false)
-#ifdef ENABLE_HIGHLIGHT
-  , _highlight(-1)
-#endif
-{
+  _overwrite(false) {
   _label.set_text(num);
-  std::string dummy;
-  create(dummy);
+  if (!create()) {
+    _ok = false;
+  }
 }
 
 Document::Document(Conf& conf, Encodings& encodings, int encoding, std::string& file) :
@@ -95,11 +102,7 @@ Document::Document(Conf& conf, Encodings& encodings, int encoding, std::string&
   _line_numbers(false),
   __on_move_cursor(0),
   __on_toggle_overwrite(0),
-  _overwrite(false)
-#ifdef ENABLE_HIGHLIGHT
-  , _highlight(-1)
-#endif
-{
+  _overwrite(false) {
   std::string contents;
 
   if (Glib::file_test(file, Glib::FILE_TEST_IS_DIR)) {
@@ -131,15 +134,33 @@ Document::Document(Conf& conf, Encodings& encodings, int encoding, std::string&
     return;
   }
   _encoding = enc;
-  create(enc == _encodings.utf8() ? contents : contents2);
-  _label.set_text(Glib::filename_to_utf8(Glib::path_get_basename(file)));
+  if (!create(enc == _encodings.utf8() ? contents : contents2)) {
+    _ok = false;
+    return;
+  }
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  try {
+    _label.set_text(Glib::filename_to_utf8(Glib::path_get_basename(file)));
+  }
+  catch (Glib::ConvertError& err) {
+    _label.set_text(Glib::filename_display_basename(file));
+  }
+#else
+  std::auto_ptr<Glib::Error> error;
+  _label.set_text(Glib::filename_to_utf8(Glib::path_get_basename(file), error));
+  if (error.get()) {
+    _label.set_text(Glib::filename_display_basename(file));
+  }
+#endif
+
   _ok = true;
   set_modified(false);
 
 #ifdef ENABLE_HIGHLIGHT
   // auto highlight.
   if (conf.get("highlight_auto", true)) {
-    set_highlight(_conf.highlight_manager.get_language(file));
+    set_highlight(SourceManager::get_language_for_file(file));
   }
 #endif
 }
@@ -153,11 +174,7 @@ Document::Document(Conf& conf, Encodings& encodings, int num, int encoding) :
   _line_numbers(false),
   __on_move_cursor(0),
   __on_toggle_overwrite(0),
-  _overwrite(false)
-#ifdef ENABLE_HIGHLIGHT
-  , _highlight(-1)
-#endif
-{
+  _overwrite(false) {
   // TODO: Bad, We are reading character by character.
   std::string contents;
   char ch;
@@ -174,7 +191,11 @@ Document::Document(Conf& conf, Encodings& encodings, int num, int encoding) :
     return;
   }
   _encoding = enc;
-  create(enc == _encodings.utf8() ? contents : contents2);
+  if (!create(enc == _encodings.utf8() ? contents : contents2)) {
+    _ok = false;
+    return;
+  }
+
   _label.set_text(num);
   _ok = true;
   set_modified(true);
@@ -190,6 +211,14 @@ Document::~Document() {
   if (__on_toggle_overwrite) {
     g_signal_handler_disconnect(G_OBJECT(_text_view.gobj()), __on_toggle_overwrite);
   }
+
+  // let's remove our temporary file.
+  std::string error;
+  if (!Utils::unlock_file(_tmp_file_fd, error)) {
+    std::cerr << "Failed to unlock " << _tmp_file << " " << error << std::endl;
+  }
+  close(_tmp_file_fd);
+  unlink(_tmp_file.c_str());
 }
 
 void Document::connect_signals() {
@@ -247,31 +276,54 @@ int Document::get_line_count() {
   return _text_view.get_buffer()->get_char_count() ? _text_view.get_buffer()->get_line_count() : 0;
 }
 
-void Document::create(std::string& str) {
-  create();
+bool Document::create(const std::string& str) {
+  if (!Utils::create_recovery_file(_tmp_file, _tmp_file_fd)) {
+    katoob_error(Utils::substitute(_("Failed to create the autorecovery file %s"), _tmp_file));
+    return false;
+  }
+
+  std::string error;
+  if (!Utils::lock_file(_tmp_file_fd, error)) {
+    katoob_error(Utils::substitute("Failed to lock the temporary file: %s", error));
+    return false;
+  }
+
+  create_ui();
   connect_signals();
+
+  if (str.size() == 0) {
+    return true;
+  }
+
   // We must connect the signals first so that the spell checker can create its lines
   // We will block do_undo
   if (do_undo) {
     do_undo = false;
-    set_text(str);
+    set_text(const_cast<std::string&>(str));
     do_undo = true;
   }
   else {
-    set_text(str);
+    set_text(const_cast<std::string&>(str));
   }
+
+  return true;
 }
 
 void Document::set_text(std::string& str) {
   _text_view.get_buffer()->set_text(str);
 }
 
-void Document::create() {
+void Document::create_ui() {
   _text_view.signal_expose_event().connect(sigc::mem_fun(*this, &Document::expose_event_cb));
+#ifdef ENABLE_MAEMO
+  g_signal_connect(_text_view.gobj(), "tap-and-hold", G_CALLBACK(__tap_and_hold), this);
+  gtk_widget_tap_and_hold_setup(GTK_WIDGET(_text_view.gobj()), NULL, NULL, GTK_TAP_AND_HOLD_NONE);
+#endif
+
   //#ifdef ENABLE_HIGHLIGHT
   //  _text_view.set_buffer(SourceBuffer::create());
   //#else
-  _text_view.set_buffer(TextBuffer::create());
+  _text_view.set_buffer(TextBuffer::create(_conf));
   //#endif
   set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
   set_shadow_type(Gtk::SHADOW_IN);
@@ -287,7 +339,7 @@ void Document::create() {
 #endif
   grab_focus();
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   std::string error;
 
   do_spell = _conf.get("spell_check", true);
@@ -315,7 +367,11 @@ void Document::create() {
   spell_checker_connect_worker();
 
   misspelled_tag = _text_view.get_buffer()->create_tag();
+#ifndef ENABLE_MAEMO
   misspelled_tag->property_underline() = Pango::UNDERLINE_ERROR;
+#else
+  g_object_set(misspelled_tag->gobj(), "underline", PANGO_UNDERLINE_ERROR, NULL);
+#endif
 #endif
 
   // TODO: Make these configurable ?
@@ -324,6 +380,11 @@ void Document::create() {
   add(_text_view);
   show_all();
   reset_gui();
+#ifdef ENABLE_MAEMO
+  // Hide the Input Method menu. We put this here so GtkSettings will have the property already.
+  g_object_set(Gtk::Settings::get_default()->gobj(), "gtk-show-input-method-menu", FALSE, NULL);
+#endif
+
 }
 
 void Document::grab_focus() {
@@ -383,6 +444,11 @@ bool Document::save(std::string& ofile, int enc, bool replace) {
       if (replace) {
 	set_readonly(false);
 	set_file(ofile);
+
+	// NOTE: We are doing it manually without calling set_encoding() because it will also mark
+	// the Document as modified but it's not.
+	_encoding = enc;
+	signal_encoding_changed.emit(enc);
       }
       return true;
     }
@@ -399,7 +465,25 @@ bool Document::save(std::string& ofile, int enc, bool replace) {
 
 void Document::set_file(std::string& nfile) {
   _file = nfile;
-  std::string f = Glib::filename_to_utf8(Glib::path_get_basename(nfile));
+
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+  std::string f;
+  try {
+    f = Glib::filename_to_utf8(Glib::path_get_basename(nfile));
+  }
+  catch (...) {
+    // We shouldn't be here.
+    f = Glib::filename_display_basename(nfile);
+  }
+#else
+  std::auto_ptr<Glib::Error> error;
+  std::string f = Glib::filename_to_utf8(Glib::path_get_basename(nfile), error);
+  if (error.get()) {
+  // We should not be getting here anyway but just in case ?
+    f = Glib::filename_display_basename(nfile);
+  }
+#endif
+
   _label.set_text(f);
   signal_file_changed.emit(nfile);
   signal_title_changed.emit(f);
@@ -411,7 +495,7 @@ void Document::on_insert(const Gtk::TextBuffer::iterator& iter , const Glib::ust
     undo(KATOOB_DO_INSERT, str, Glib::RefPtr<TextBuffer>::cast_dynamic(_text_view.get_buffer())->get_mark_insert_position());
   }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   spell_checker_on_insert(iter, len);
   spell_checker_connect_worker();
 #endif
@@ -427,7 +511,7 @@ void Document::on_erase(const Gtk::TextBuffer::iterator& start, const Gtk::TextB
     b->clear_deleted();
   }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   spell_checker_on_erase(start, end);
   spell_checker_connect_worker();
 #endif
@@ -469,11 +553,7 @@ int Document::calculate_column(Gtk::TextIter& iter) {
   return column;
 }
 
-void Document::insert(std::string& str) {
-  _text_view.get_buffer()->insert_at_cursor(str);
-}
-
-void Document::insert(Glib::ustring& str) {
+void Document::insert(const std::string& str) {
   _text_view.get_buffer()->insert_at_cursor(str);
 }
 
@@ -785,7 +865,7 @@ void Document::emit_signals() {
   signal_overwrite_toggled.emit(_overwrite);
   on_move_cursor();
   signal_title_changed.emit(get_title());
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   signal_auto_spell_set.emit(do_spell);
   signal_dictionary_changed.emit(spell_dict);
 #endif
@@ -1162,7 +1242,7 @@ void Document::on_populate_popup_cb(Gtk::Menu *menu) {
     item = &menu->items().back();
     item->signal_activate().connect(sigc::bind<std::string>(sigc::mem_fun(*this, &Document::dict_menu_item_activated), word));
   }
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   if (get_readonly()) {
     return;
   }
@@ -1214,7 +1294,7 @@ void Document::on_populate_popup_cb(Gtk::Menu *menu) {
 #endif
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void Document::spell_checker_connect_worker() {
   if (do_spell) {
     if (!spell_worker_conn.connected()) {
@@ -1661,19 +1741,19 @@ void Document::reset_gui() {
 }
 
 #ifdef ENABLE_HIGHLIGHT
-void Document::set_highlight(int x) {
+void Document::set_highlight(const std::string& x) {
   if (_highlight == x) {
     return;
   }
 
   _highlight = x;
 
-  if (x == -1) {
+  if (_highlight == "") {
     // none
     Glib::RefPtr<TextBuffer>::cast_dynamic(_text_view.get_buffer())->set_highlight(false);
   }
   else {
-    Glib::RefPtr<TextBuffer>::cast_dynamic(_text_view.get_buffer())->set_language(_conf.highlight_manager.get_language(x));
+    Glib::RefPtr<TextBuffer>::cast_dynamic(_text_view.get_buffer())->set_language(SourceManager::get_language(_highlight));
     Glib::RefPtr<TextBuffer>::cast_dynamic(_text_view.get_buffer())->set_highlight(true);
   }
 }
@@ -1703,3 +1783,24 @@ void Document::set_tab_width() {
 
   _text_view.set_tabs(tab_array);
 }
+
+void Document::autosave() {
+  if (ftruncate(_tmp_file_fd, 0) == -1) {
+    std::cerr << "Failed to truncate temp file: " << std::strerror(errno) << std::endl;
+    return;
+  }
+
+  if (lseek(_tmp_file_fd, 0, SEEK_SET) != 0) {
+    std::cerr << "Failed to seek in temp file: " << std::strerror(errno) << std::endl;
+  }
+
+  if (write(_tmp_file_fd, _text_view.get_buffer()->get_text().c_str(), _text_view.get_buffer()->get_text().size()) == -1) {
+    std::cerr << "Failed to write to temp file: " << std::strerror(errno) << std::endl;
+  }
+}
+
+#ifdef ENABLE_MAEMO
+void Document::_tap_and_hold() {
+  g_signal_emit_by_name(GTK_WIDGET(_text_view.gobj()), "popup-menu");
+}
+#endif
