@@ -32,7 +32,7 @@
 #include "emulator.hh"
 #include "utils.hh"
 
-Emulator::Emulator() : _ok(false), layout(-1) {
+Emulator::Emulator() {
   std::string dir(Utils::get_data_dir() + Utils::get_dir_separator() + "emulator" + Utils::get_dir_separator());
 
   std::auto_ptr<Glib::Dir> d;
@@ -47,10 +47,14 @@ Emulator::Emulator() : _ok(false), layout(-1) {
 
   Glib::DirIterator start = d->begin();
   Glib::DirIterator end = d->end();
+#ifndef GLIBMM_EXCEPTIONS_ENABLED
+  std::auto_ptr<Glib::Error> error;
+#endif
   while (start != end) {
     std::map<std::string, std::string> map;
     std::string file = dir + *start;
     if (parse_file(file, map)) {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
       try {
 	std::string _utf8 = Glib::filename_to_utf8(*start);
 	names.push_back(_utf8);
@@ -58,6 +62,16 @@ Emulator::Emulator() : _ok(false), layout(-1) {
       catch (Glib::ConvertError& er) {
 	names.push_back(*start);
       }
+#else
+      std::string _utf8 = Glib::filename_to_utf8(*start, error);
+      if (error.get()) {
+	names.push_back(*start);
+	error.reset();
+      }
+      else {
+	names.push_back(_utf8);
+      }
+#endif
       layouts.push_back(map);
       _ok = true;
     }
@@ -97,7 +111,7 @@ bool Emulator::parse_file(std::string& file, std::map<std::string, std::string>&
   }
 }
 
-bool Emulator::get(std::string& key, std::string& value) {
+bool Emulator::get(const std::string& key, std::string& value) {
   assert(layout < layouts.size());
   assert(layout >= 0);
 
@@ -122,3 +136,9 @@ std::map<std::string, std::string>& Emulator::get_layout() {
   assert(layout != -1);
   return layouts[layout];
 }
+
+std::vector<std::string> Emulator::names;
+std::vector<std::map<std::string, std::string> > Emulator::layouts;
+bool Emulator::_ok = false;
+int Emulator::layout = -1;
+std::string Emulator::_err;
