@@ -2,7 +2,7 @@
  * encodings.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -337,6 +337,7 @@ bool Encodings::convert(const Glib::ustring& text, std::string& res, unsigned in
   assert(from <= _encodings.size());
   assert(to <= _encodings.size());
   assert(from != to);
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try {
     res = Glib::convert (text, get_charset(to), get_charset(from));
     if (utf8(res)) {
@@ -351,6 +352,22 @@ bool Encodings::convert(const Glib::ustring& text, std::string& res, unsigned in
     err = _err.what();
     return false;
   }
+#else
+  std::auto_ptr<Glib::Error> error;
+  res = Glib::convert (text, get_charset(to), get_charset(from), error);
+  if (error.get()) {
+    err = error->what();
+    return false;
+  }
+
+  if (utf8(res)) {
+    return true;
+  }
+  else {
+    err = _("I wasn't able to convert the encoding.");
+    return false;
+  }
+#endif
 }
 
 /**
