@@ -2,7 +2,7 @@
  * filedialog.cc
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,24 +29,34 @@
 #include "macros.h"
 #include "dialogs.hh"
 
-FileDialog::FileDialog(const std::string& title, Gtk::FileChooserAction action, int enc, Conf& _conf, Encodings& _enc) : Gtk::FileChooserDialog(title, action), _encodings(_enc) {
-  // TODO: We want to handle non-local only oneday.
-  set_local_only(true);
+FileDialog::FileDialog(const std::string& title, Gtk::FileChooserAction action, int enc, Conf& _conf, Encodings& _enc) : SimpleFileDialog(title, action, _conf), _encodings(_enc) {
 
   if (action == FILE_OPEN) {
     set_select_multiple(true);
   }
-
+#ifndef ENABLE_MAEMO
   label.set_text(_("Encoding"));
   box.pack_start(label);
   label.set_padding (20, 0);
+#endif
   for (int x = 0; x < _enc.size(); x++) {
     if (((action == FILE_OPEN) && x != _enc.utf8()) || (action == FILE_SAVE)) {
       cbox.append_text(_enc.at(x));
     }
   }
+#ifndef ENABLE_MAEMO
   box.pack_start (cbox);
+#endif
+
+#ifdef ENABLE_MAEMO
+  caption = new Hildon::Caption(_("Encoding"), cbox);
+  add_extra(*caption);
+  // I must call it otherwise it won't show.
+  caption->show_all();
+#else
   set_extra_widget(box);
+  box.show_all();
+#endif
 
   if ((action == FILE_OPEN) && (enc == -1)) {
     enc = _enc.default_open();
@@ -63,20 +73,12 @@ FileDialog::FileDialog(const std::string& title, Gtk::FileChooserAction action,
   }
 
   cbox.set_active(enc);
-
-  /*
-//  gtk_window_set_transient_for (GTK_WINDOW (file_selector),
-  //                              GTK_WINDOW (katoob->win));
-*/
-
-  add_button(Gtk::Stock::OK, 1);
-  add_button(Gtk::Stock::CANCEL, 0);
-
-  show_all();
 }
 
 FileDialog::~FileDialog() {
-
+#ifdef ENABLE_MAEMO
+  delete caption;
+#endif
 }
 
 int FileDialog::encoding() {
@@ -86,7 +88,11 @@ int FileDialog::encoding() {
 }
 
 bool FileDialog::run() {
-  bool st = Gtk::FileChooserDialog::run();
+#ifdef ENABLE_MAEMO
+  bool st = Hildon::FileChooserDialog::run() == Gtk::RESPONSE_OK;
+#else
+  bool st = Gtk::FileChooserDialog::run() == Gtk::RESPONSE_OK;
+#endif
   hide();
   return st;
 }
@@ -95,8 +101,12 @@ std::vector<std::string> FileDialog::get() {
   return get_filenames();
 }
 
-
-SimpleFileDialog::SimpleFileDialog(const std::string& title, Gtk::FileChooserAction action, Conf& _conf) : Gtk::FileChooserDialog(title, action) {
+SimpleFileDialog::SimpleFileDialog(const std::string& title, Gtk::FileChooserAction action, Conf& _conf) :
+#ifdef ENABLE_MAEMO
+  Hildon::FileChooserDialog(action) {
+#else
+  Gtk::FileChooserDialog(title, action) {
+#endif
   // TODO: We want to handle non-local only oneday.
   set_local_only(true);
   if (action == FILE_OPEN) {
@@ -105,9 +115,15 @@ SimpleFileDialog::SimpleFileDialog(const std::string& title, Gtk::FileChooserAct
   else {
     set_current_folder(_conf.save_dir());
   }
-  add_button(Gtk::Stock::OK, 1);
-  add_button(Gtk::Stock::CANCEL, 0);
-
+#ifndef ENABLE_MAEMO
+  add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  if (action == FILE_OPEN) {
+  add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
+  }
+  else {
+    add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
+  }
+#endif
   show_all();
 }
 
@@ -116,7 +132,11 @@ SimpleFileDialog::~SimpleFileDialog() {
 }
 
 bool SimpleFileDialog::run() {
-  bool st = Gtk::FileChooserDialog::run();
+#ifdef ENABLE_MAEMO
+  bool st = Hildon::FileChooserDialog::run() == Gtk::RESPONSE_OK;
+#else
+  bool st = Gtk::FileChooserDialog::run() == Gtk::RESPONSE_OK;
+#endif
   hide();
   if (st) {
     std::string f = get();
