@@ -2,7 +2,7 @@
  * macros.h
  * This file is part of katoob
  *
- * Copyright (C) 2006 Mohammed Sameer
+ * Copyright (C) 2006, 2007 Mohammed Sameer
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,6 +43,9 @@
 #define N_(String) (String)
 #endif /* ENABLE_NLS */
 
-#define REMOTE_COMMAND "wget -q --passive-ftp -O - \"%s\""
+#define TABS_POS_TOP                    101
+#define TABS_POS_BOTTOM                 102
+#define TABS_POS_RIGHT                  103
+#define TABS_POS_LEFT                   104
 
 #endif /* __MACROS_H__ */
