@@ -24,19 +24,20 @@
 #include <config.h>
 #endif /* HAVE_CONFIG_H */
 
+#include <cerrno>
 #include <iostream>
 #include "mdi.hh"
 #include "dialogs.hh"
 #include "filedialog.hh"
 #include "macros.h"
-#ifdef ENABLE_PRINT
-#include "printdialog.hh"
-#endif
 #include "searchdialog.hh"
 //#include "replacedialog.hh"
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 #include "spelldialog.hh"
 #endif
+#ifdef ENABLE_PRINT
+#include "print.hh"
+#endif
 #include "dict.hh"
 #include "utils.hh"
 #include <cassert>
@@ -44,14 +45,74 @@
 #include "pipe.hh"
 #include "tempfile.hh"
 #include "openlocationdialog.hh"
-#include "thread.hh"
+#include "network.hh"
 
 MDI::MDI(Conf& conf, Encodings& enc) :
-  _conf(conf), _encodings(enc) {
+  _conf(conf),
+  _encodings(enc)
+#ifdef ENABLE_PRINT
+  , page_setup(PageSetup::create(_conf)),
+  settings(PrintSettings::create(_conf))
+#endif
+{
   signal_switch_page().connect(sigc::mem_fun(*this, &MDI::signal_switch_page_cb));
+  // 1 minute.
+  Glib::signal_timeout().connect(sigc::mem_fun(this, &MDI::autosave), 1*60*1000);
 }
 
 MDI::~MDI() {
+  // destroy our documents.
+  for (unsigned x = 0; x < children.size(); x++) {
+    delete children[x];
+  }
+
+  for (unsigned x = 0; x < closed_children.size(); x++) {
+    delete closed_children[x];
+  }
+
+  children.clear();
+  closed_children.clear();
+  page_setup->save();
+  settings->save();
+}
+
+void MDI::scan_temp() {
+  std::map<std::string, std::string> temps;
+  std::string error;
+  if (!Utils::get_recovery_files(temps, error)) {
+    katoob_error(Utils::substitute(_("Failed to scan for any autorecovery files: %s"), error));
+    return;
+  }
+
+  if (temps.size() > 0) {
+    if (katoob_simple_question(_("Some unrecovered files were found. Try to recover them ?"))) {
+      for (std::map<std::string, std::string>::iterator iter = temps.begin(); iter != temps.end(); iter++) {
+	Document *doc = create_document();
+	if (doc) {
+	  doc->set_text(iter->second);
+	  if (unlink(iter->first.c_str()) == -1) {
+	    std::cerr << "Failed to unlink " << iter->first << " " << std::strerror(errno) << std::endl;
+	  }
+	}
+      }
+    }
+  }
+}
+
+bool MDI::autosave() {
+  Document *doc;
+  for (unsigned x = 0; x < children.size(); x++) {
+    doc = children[x];
+    if (doc->get_readonly()) {
+      continue;
+    }
+    if (!doc->get_modified()) {
+      continue;
+    }
+    doc->autosave();
+  }
+
+  return true;
 }
 
 Document *MDI::create_document(std::string& file, int enc) {
@@ -96,8 +157,14 @@ Document *MDI::create_document(std::string& file, int enc) {
 Document *MDI::create_document() {
   int x = children.size();
   Document *doc = new Document(_conf, _encodings, ++x);
-  add_document(doc);
-  return doc;
+  if (doc->ok()) {
+    add_document(doc);
+    return doc;
+  }
+  else {
+    delete doc;
+    return NULL;
+  }
 }
 
 void MDI::add_document(Document *doc, bool signals) {
@@ -154,33 +221,54 @@ void MDI::open_location_cb() {
 
   bool to_active = false;
 
-  std::string out;
-  if(dialog.run(out)) {
+  std::string error;
+  if(dialog.run(error)) {
     if (doc) {
       to_active = dialog.insert_to_active();
     }
     // get encoding,
     int enc = dialog.get_encoding();
-    if (!to_active) {
+    std::string uri = dialog.get_uri();
+    sigc::slot<void, bool, const std::string&> slot = sigc::bind(sigc::mem_fun(this, &MDI::signal_transfer_complete_cb), uri, enc, to_active);
+    std::string error;
+    if (!Network::add_transfer(uri, error, slot)) {
+      katoob_error(error);
+    }
+  }
+  else if (error.size() > 0) {
+    katoob_error(error);
+  }
+}
+
+void MDI::signal_transfer_complete_cb(bool st, const std::string& str, const std::string uri, int enc,  bool to_active) {
+  if (!st) {
+    katoob_error(str);
+    return;
+  }
+
+  // We convert it to utf8.
+  std::string res;
+  if (_encodings.convert_to(str, res, enc) != -1) {
+    Document *doc = get_active();
+    if (!doc) {
       doc = create_document();
     }
-    // We convert it to utf8.
-    std::string out2;
-    if (_encodings.convert_to(out, out2, enc) != -1) {
-      // Insert.
-      doc->insert(out2.size() == 0 ? out : out2);
+    else if (!to_active) {
+      doc = create_document();
+    }
+    // Insert.
+    if (doc) {
       // TODO: Set a title for the document.
       // We also need the document to know that it's a remote document.
-    }
-    else {
-      katoob_error(out2);
+      doc->insert(res);
     }
   }
   else {
-    if (out.size() > 0) {
-      katoob_error(out);
-    }
+    katoob_error(res);
   }
+
+  //  std::cout << uri << std::endl;
+  //std::cout << str << std::endl;
 }
 
 void MDI::insert_file_cb() {
@@ -309,52 +397,34 @@ void MDI::print_cb() {
     return;
   }
 
-  PrintDialog dialog(_conf);
-
-  std::string err;
-  if (!dialog.ok(err)) {
-    katoob_error(err);
-    return;
-  }
-
-  dialog.has_selection(doc->has_selection());
-  dialog.lines(doc->get_line_count());
-
-  if (!dialog.run()) {
-    return;
-  }
-
-  dialog.start_process();
-
-  std::vector<std::string> lines;
-  if (dialog.all()) {
-    doc->get_lines(lines);
-  }
-  else if (dialog.selection()) {
-    doc->get_selection(lines);
+  Glib::RefPtr<Print> prnt = Print::create(_conf, doc, page_setup, settings);
+  std::string error;
+  if (!prnt->run(error, Gtk::PRINT_OPERATION_ACTION_PRINT_DIALOG)) {
+    if (error.size() > 0) {
+      katoob_error(error);
+    }
   }
   else {
-    int x, y;
-    bool l = dialog.range(x, y);
-    assert(x < y);
-    assert(l = true);
-    if (!l) {
-      katoob_error(_("You must select something to print."));
-      return;
-    }
-    doc->get_lines(lines, x, y);
+    // TODO: Save PrintSettings
   }
+}
 
-  for (unsigned l = 0; l < lines.size(); l++) {
-    dialog.add_line(lines[l]);
-  }
-
-  if (!dialog.end_process(err)) {
-    katoob_error(err);
+void MDI::print_preview_cb() {
+  Document *doc = get_active();
+  if (!doc) {
     return;
   }
 
-  dialog.manipulate();
+  Glib::RefPtr<Print> prnt = Print::create(_conf, doc, page_setup, settings);
+  std::string error;
+  if (!prnt->run(error, Gtk::PRINT_OPERATION_ACTION_PREVIEW)) {
+    katoob_error(error);
+  }
+}
+
+void MDI::page_setup_cb() {
+  // NOTE: All this so we can get our damn parent!
+  page_setup->reset(Gtk::run_page_setup_dialog(*(dynamic_cast<Gtk::Window *>(get_parent()->get_parent())), page_setup->get_page_setup(), settings));
 }
 #endif
 
@@ -411,6 +481,7 @@ bool MDI::close(int n) {
   // Let's add it to our closed documents vector.
   // Is it enabled ?
   if (_conf.get("undo_closed", true)) {
+    doc->autosave();
     closed_children.push_back(doc);
     // Now let's see how many items do we have.
     unsigned x = _conf.get("undo_closedno", 5);
@@ -433,6 +504,12 @@ bool MDI::close(int n) {
 }
 
 void MDI::recent_cb(std::string& rfile) {
+  // fail if the file is not there.
+  if (!Glib::file_test(rfile.c_str(), Glib::FILE_TEST_EXISTS)) {
+    katoob_error(Utils::substitute(_("The file \"%s\" doesn't exist."), rfile.c_str()));
+    return;
+  }
+
   unsigned x = children.size();
   create_document(rfile, -1);
   if (children.size() != x) {
@@ -701,7 +778,7 @@ void MDI::replace_dialog_signal_replace_all_cb(ReplaceDialog *dialog) {
     ++x;
     doc->replace();
   }
-  katoob_info(Utils::substitute(_("Replaced %d occurences."), x));
+  katoob_info(Utils::substitute(ngettext("Replaced %d occurence.", "Replaced %d occurences.", x), x));
 }
 
 void MDI::execute_cb() {
@@ -764,12 +841,14 @@ void MDI::execute_cb() {
     else {
       // We create a new document then we insert.
       Document *d = create_document();
-      d->set_text(text);
+      if (d) {
+	d->set_text(text);
+      }
     }
   }
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void MDI::set_auto_spell(bool s) {
   Document *doc = get_active();
   if (!doc) {
@@ -822,7 +901,7 @@ void MDI::connect_signals(Document *doc) {
   doc->signal_encoding_changed.connect(sigc::mem_fun(this, &MDI::signal_document_encoding_changed_cb));
   doc->signal_overwrite_toggled.connect(sigc::mem_fun(this, &MDI::signal_document_overwrite_toggled_cb));
   doc->signal_title_changed.connect(sigc::mem_fun(this, &MDI::signal_document_title_changed_cb));
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   doc->signal_auto_spell_set.connect(sigc::mem_fun(this, &MDI::signal_document_spell_enabled_cb));
   doc->signal_dictionary_changed.connect(sigc::mem_fun(this, &MDI::signal_dictionary_changed_cb));
 #endif
@@ -858,32 +937,43 @@ bool MDI::set_encoding(int n, int& o) {
 }
 
 void MDI::signal_document_dict_lookup_cb(std::string word) {
-  std::string result, error;
-  try {
-    Dispatcher *dispatcher = new Dispatcher;
-    dispatcher->in = word;
+  std::string uri = Dict::construct_uri(_conf, word);
 
-    sigc::slot<void> slot = sigc::bind<Conf *, Dispatcher *>(sigc::ptr_fun(katoob_dict_define_word), &_conf, dispatcher);
-    Thread thread(slot, dispatcher);
+  sigc::slot<void, bool, const std::string&> slot = sigc::bind(sigc::mem_fun(this, &MDI::signal_dict_transfer_complete_cb), word);
+  std::string error;
+  if (!Network::add_transfer(uri, error, slot)) {
+    katoob_error(error);
+  }
+}
 
-    std::string error;
-    if (thread.ok(error)) {
-      if (dispatcher->err) {
-	katoob_error(dispatcher->error);
-      }
-      else {
-	katoob_info(dispatcher->out);
-      }
+void MDI::signal_dict_transfer_complete_cb(bool st, const std::string& str, std::string word) {
+  if (!st) {
+    katoob_error(str);
+    return;
+  }
+
+  std::vector<std::string> defs;
+
+  if (!Dict::parse_defs(str, defs)) {
+    if (defs.size() == 0) {
+      katoob_error(_("Failed to get any definitions."));
+    }
+    else if (defs[0].size() == 0) {
+      katoob_error(_("Failed to get any definitions."));
     }
     else {
-      katoob_error(error);
+      katoob_error(defs[0]);
     }
-    delete dispatcher;
   }
-  catch (Glib::FileError &er) {
-#if ((defined(_WIN32) && !defined(NDEBUG)) || (!defined(_WIN32)))
-    std::cout << er.what() << std::endl;
-#endif
+  else {
+    // TODO: only one definition is being displayed.
+    Document *doc = get_active();
+    if (katoob_big_info(defs[0], doc != NULL)) {
+      // not needed but just in case!
+      if (doc) {
+	doc->insert(defs[0]);
+      }
+    }
   }
 }
 
@@ -933,8 +1023,13 @@ void MDI::reset_gui() {
     popup_disable();
   }
 
+#ifdef ENABLE_MAEMO
+  set_show_tabs(_conf.get("showtabs", false));
+  set_scrollable(_conf.get("scrolltabs", false));
+#else
   set_show_tabs(_conf.get("showtabs", true));
   set_scrollable(_conf.get("scrolltabs", true));
+#endif
 
   switch(_conf.get("tabspos", TABS_POS_TOP)) {
   case TABS_POS_BOTTOM:
@@ -974,6 +1069,10 @@ void MDI::reset_gui() {
       dynamic_cast<Label *>(get_tab_label(*children[x]))->set_normal();
     }
   }
+
+#ifdef ENABLE_PRINT
+  settings->reset();
+#endif
 }
 
 void MDI::import_cb(Import im) {
@@ -991,14 +1090,18 @@ void MDI::import_cb(Import im) {
   if (_encodings.utf8(out)) {
     // create.
     Document *doc = create_document();
-    doc->set_text(out);
+    if (doc) {
+      doc->set_text(out);
+    }
   }
   else {
     std::string res;
     if (_encodings.convert_to(out, res, _encodings.default_open()) != -1) {
       // create.
       Document *doc = create_document();
-      doc->set_text(res);
+      if (doc) {
+	doc->set_text(res);
+      }
     }
     else {
       katoob_error(Utils::substitute(_("Couldn't detect the encoding of %s"), file));
@@ -1061,7 +1164,7 @@ void MDI::signal_extra_button_clicked_cb(std::string str) {
 }
 
 #ifdef ENABLE_HIGHLIGHT
-void MDI::set_highlight(int x) {
+void MDI::set_highlight(std::string x) {
   Document *doc = get_active();
   if (doc) {
     doc->set_highlight(x);
