@@ -31,17 +31,37 @@
 #include "menubar.hh"
 #include "macros.h"
 #include "utils.hh"
+#ifdef ENABLE_HIGHLIGHT
+#include "sourcemanager.hh"
+#endif
 #include <cassert>
+#include <libgen.h>
+#ifdef ENABLE_MAEMO
+#include <gtkmm/settings.h>
+#endif
 
-// TODO: recent should display the file name only.
 // TODO: Integrate with the gtk recent files thing.
-MenuBar::MenuBar(Conf& config, Encodings& encodings, std::vector<std::string>& em, std::vector<std::string>& mp) :
+MenuBar::MenuBar(Conf& config, Encodings& encodings
+#ifdef ENABLE_EMULATOR
+		 , std::vector<std::string>& em
+#endif
+#ifdef ENABLE_MULTIPRESS
+		 , std::vector<std::string>& mp
+#endif
+		 ) :
   _conf(config) {
   file(config);
   edit(config);
   search(config);
   view(config, encodings);
-  tools(config, em, mp);
+  tools(config
+#ifdef ENABLE_EMULATOR
+	, em
+#endif
+#ifdef ENABLE_MULTIPRESS
+	, mp
+#endif
+	);
   documents(config);
   help(config);
 #ifdef ENABLE_HIGHLIGHT
@@ -49,6 +69,11 @@ MenuBar::MenuBar(Conf& config, Encodings& encodings, std::vector<std::string>& e
 #endif
   show_all();
   recent_menu_item->hide();
+#ifdef ENABLE_MAEMO
+  // NOTE: The only way to hide the icons is here. We'll have generated GtkImageMenuItems by
+  // now and GtkSettings will carry the "gtk-menu-images" option.
+  g_object_set(Gtk::Settings::get_default()->gobj(), "gtk-menu-images", FALSE, NULL);
+#endif
 }
 
 MenuBar::~MenuBar() {
@@ -97,6 +122,7 @@ Gtk::MenuItem *MenuBar::item(Gtk::Menu *menu, const std::string& str) {
 }
 
 Gtk::MenuItem *MenuBar::item(Gtk::Menu *menu, const Gtk::StockID& stock_id, guint key, Gdk::ModifierType mod) {
+  // TODO: Missing shortcuts under maemo.
   Gtk::AccelKey _key(key, mod);
   menu->items().push_back(Gtk::Menu_Helpers::StockMenuElem(stock_id, _key));
   return &menu->items().back();
@@ -156,12 +182,19 @@ void MenuBar::file(Conf& config) {
   for (unsigned x = 0; x < exports.size(); x++) {
     item(_export_menu, exports[x].name)->signal_activate().connect(sigc::bind<Export>(sigc::mem_fun(this, &MenuBar::signal_export_activate_cb), exports[x]));
   }
-  separator(file_menu);
+
 #ifdef ENABLE_PRINT
+  separator(file_menu);
+  _page_setup = item(file_menu, _("Page Set_up..."));
+  _page_setup->signal_activate().connect(sigc::mem_fun(signal_page_setup_activate, &sigc::signal<void>::emit));
+
+  _print_preview = item(file_menu, _("Print Previe_w"), GDK_p, Gdk::ModifierType(GDK_CONTROL_MASK|GDK_SHIFT_MASK));
+  _print_preview->signal_activate().connect(sigc::mem_fun(signal_print_preview_activate, &sigc::signal<void>::emit));
+
   _print = item(file_menu, Gtk::Stock::PRINT, GDK_p, Gdk::ModifierType(GDK_CONTROL_MASK));
   _print->signal_activate().connect(sigc::mem_fun(signal_print_activate, &sigc::signal<void>::emit));
-  separator(file_menu);
 #endif
+  separator(file_menu);
   _close = item(file_menu, Gtk::Stock::CLOSE);
   _close->signal_activate().connect(sigc::mem_fun(signal_close_activate, &sigc::signal<void>::emit));
 
@@ -181,8 +214,25 @@ void MenuBar::create_recent() {
   recent_menu->items().clear();
   unsigned recentno = _conf.get("recentno", 10);
   recentno = recentno > _conf.get_recent().size() ? _conf.get_recent().size() : recentno;
+  std::string base;
+
   for (unsigned x = 0; x < recentno; x++) {
-    Gtk::MenuItem *_item = item(recent_menu, Glib::filename_to_utf8(_conf.get_recent()[x]));
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
+    try {
+      base = Glib::filename_to_utf8(Glib::path_get_basename(_conf.get_recent()[x]));
+    }
+    catch (...) {
+      base = Glib::filename_display_basename(_conf.get_recent()[x]);
+    }
+#else
+    // Note: We will skip non valid files for now.
+    std::auto_ptr<Glib::Error> error;
+    base = Glib::filename_to_utf8(Glib::path_get_basename(_conf.get_recent()[x]), error);
+    if (error.get()) {
+      base = Glib::filename_display_basename(_conf.get_recent()[x]);
+    }
+#endif
+    Gtk::MenuItem *_item = item(recent_menu, base);
     _item->signal_activate().connect(sigc::bind<std::string>(sigc::mem_fun(*this, &MenuBar::signal_recent_activate_cb), _conf.get_recent()[x]));
   }
 }
@@ -292,36 +342,53 @@ void MenuBar::view(Conf& config, Encodings& encodings) {
     }
 }
 
-void MenuBar::tools(Conf& config, std::vector<std::string>& em, std::vector<std::string>& mp) {
+void MenuBar::tools(Conf& config
+#ifdef ENABLE_EMULATOR
+		    , std::vector<std::string>& em
+#endif
+#ifdef ENABLE_MULTIPRESS
+		    , std::vector<std::string>& mp
+#endif
+) {
   tools_menu = menu(_("_Tools"));
   _execute = item(tools_menu, _("_Execute Command On Buffer..."), GDK_e, Gdk::ModifierType(GDK_CONTROL_MASK));
   _execute->signal_activate().connect(sigc::mem_fun(signal_execute_activate, &sigc::signal<void>::emit));
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS) || defined(ENABLE_SPELL)
   separator(tools_menu);
-#ifdef HAVE_SPELL
+#endif
+#ifdef ENABLE_SPELL
   _spell = item(tools_menu, Gtk::Stock::SPELL_CHECK, GDK_F7, Gdk::ModifierType(GDK_CONTROL_MASK));
   _spell->signal_activate().connect(sigc::mem_fun(signal_spell_activate, &sigc::signal<void>::emit));
 
   _auto_spell = check_item(tools_menu, _("_Autocheck Spelling"));
   _auto_spell->signal_activate().connect(sigc::mem_fun(*this, &MenuBar::signal_auto_spell_activate_cb));
 #endif
+
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
   Gtk::RadioButtonGroup input_group;
   _input_menu = menu(_("Input"), tools_menu);
   Gtk::MenuItem *__item = radio_item(_input_menu, input_group, _("Default"));
   dynamic_cast<Gtk::RadioMenuItem *>(__item)->set_active(true);
   __item->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &MenuBar::signal_layout_activate_cb), -1, -1));
-
+#endif
+#ifdef ENABLE_EMULATOR
   _emulator_menu = menu(_("Keyboard emulator"), _input_menu);
-  _multipress_menu = menu(_("Multipress"), _input_menu);
   build_submenu(_emulator_menu, em, input_group, 0);
+#endif
+#ifdef ENABLE_MULTIPRESS
+  _multipress_menu = menu(_("Multipress"), _input_menu);
   build_submenu(_multipress_menu, mp, input_group, 1);
+#endif
 }
 
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
 void MenuBar::build_submenu(Gtk::Menu *menu, std::vector<std::string>& items, Gtk::RadioButtonGroup& group, int num) {
   for (unsigned x = 0; x < items.size(); x++) {
       Gtk::MenuItem *_item = radio_item(menu, group, items[x]);
       _item->signal_activate().connect(sigc::bind<int, int>(sigc::mem_fun(*this, &MenuBar::signal_layout_activate_cb), num, x));
   }
 }
+#endif
 
 void MenuBar::documents(Conf& config) {
   documents_menu = menu(_("_Documents"));
@@ -344,9 +411,11 @@ void MenuBar::help(Conf& config) {
   _about->signal_activate().connect(mem_fun(signal_about_activate, &sigc::signal<void>::emit));
 }
 
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
 void MenuBar::signal_layout_activate_cb(int x, int y) {
   signal_layout_activate.emit(x, y);
 }
+#endif
 
 void MenuBar::set_encoding(unsigned e) {
   assert (e <= encoding_menu_items.size());
@@ -369,28 +438,6 @@ void MenuBar::signal_encoding_activate_cb(unsigned e) {
   if (dynamic_cast<Gtk::CheckMenuItem *>(item)->get_active()) {
     signal_encoding_activate.emit(e);
   }
-
-#if 0
-  e++;
-
-  MenuList& items = encoding_menu->items();
-  for (int x = 0; x < items.size(); x++)
-    {
-      MenuList& _items = items[x].get_submenu()->items();
-      int t = e - _items.size();
-      if (t > 0)
-	e -= _items.size();
-      else
-	{
-	  // TODO: BAD BAD BAD VERY BAD AND UGLY.
-	  if (((Gtk::RadioMenuItem *)&_items[--e])->get_active())
-	    {
-	      signal_encoding_activate.emit(e);
-	      return;
-	    }
-	}
-    }
-#endif
 }
 
 void MenuBar::signal_document_activate_cb(int x) {
@@ -575,13 +622,18 @@ void MenuBar::signal_export_activate_cb(Export exp) {
 }
 
 void MenuBar::reset_gui() {
-  dynamic_cast<Gtk::CheckMenuItem *>(_statusbar)->set_active(_conf.get("statusbar", true));
+#ifdef ENABLE_MAEMO
+  dynamic_cast<Gtk::CheckMenuItem *>(_line_numbers)->set_active(_conf.get("linenumbers", false));
+#else
   dynamic_cast<Gtk::CheckMenuItem *>(_line_numbers)->set_active(_conf.get("linenumbers", true));
+#endif
+
+  dynamic_cast<Gtk::CheckMenuItem *>(_statusbar)->set_active(_conf.get("statusbar", true));
   dynamic_cast<Gtk::CheckMenuItem *>(_wrap_text)->set_active(_conf.get("textwrap", true));
 
   dynamic_cast<Gtk::CheckMenuItem *>(_toolbar)->set_active(_conf.get("toolbar", true));
   dynamic_cast<Gtk::CheckMenuItem *>(_extended_toolbar)->set_active(_conf.get("extended_toolbar", true));
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   dynamic_cast<Gtk::CheckMenuItem *>(_auto_spell)->set_active(_conf.get("spell_check", true));
 #endif
 
@@ -613,7 +665,7 @@ void MenuBar::reset_gui() {
   document_set_active(_active);
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void MenuBar::signal_auto_spell_activate_cb() {
   signal_auto_spell_activate.emit(dynamic_cast<Gtk::CheckMenuItem *>(_auto_spell)->get_active());
 }
@@ -678,7 +730,7 @@ void MenuBar::enable_line_numbers(bool s) {
   dynamic_cast<Gtk::CheckMenuItem *>(_line_numbers)->set_active(s);
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void MenuBar::enable_auto_spell(bool s) {
   dynamic_cast<Gtk::CheckMenuItem *>(_auto_spell)->set_active(s);
 }
@@ -693,6 +745,7 @@ void MenuBar::reset_gui(bool enable) {
   _import_menu->set_sensitive(enable);
 
 #ifdef ENABLE_PRINT
+  _print_preview->set_sensitive(enable);
   _print->set_sensitive(enable);
 #endif
   _close->set_sensitive(enable);
@@ -711,11 +764,13 @@ void MenuBar::reset_gui(bool enable) {
   _wrap_text->set_sensitive(enable);
   _encoding_menu->set_sensitive(enable);
   _execute->set_sensitive(enable);
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   _spell->set_sensitive(enable);
   _auto_spell->set_sensitive(enable);
 #endif
+#ifdef ENABLE_EMULATOR
   _emulator_menu->set_sensitive(enable);
+#endif
   _save_all->set_sensitive(enable);
   _close_all->set_sensitive(enable);
 }
@@ -735,45 +790,96 @@ void MenuBar::signal_recent_activate_cb(std::string& str) {
 
 #ifdef ENABLE_HIGHLIGHT
 void MenuBar::create_highlighters() {
-  SourceManager& manager = _conf.highlight_manager;
   Gtk::Menu *item;
   Gtk::MenuItem *_item;
   Gtk::RadioButtonGroup highlighters_radio;
 
-  Gtk::Menu *highlight = menu(_("_Highlight"), view_menu);
+  highlight = menu(_("_Highlight"), view_menu);
 
   _item = radio_item(highlight, highlighters_radio, _("None"));
-  highlighters.push_back(_item);
-  _item->signal_activate().connect(sigc::bind<int>(sigc::mem_fun(*this, &MenuBar::signal_highlighter_activate_cb), -1));
-  manager.set_map(-1, 0);
+  _item->signal_activate().connect(sigc::bind<std::string>(sigc::mem_fun(*this, &MenuBar::signal_highlighter_activate_cb), ""));
 
-  SourceCategory& cats = manager.get_categories();
+  SourceCategory& cats = SourceManager::get_categories();
   SourceCategoryIter iter;
 
   for (iter = cats.begin(); iter != cats.end(); iter++) {
     item = menu(const_cast<char *>(iter->first.c_str()), highlight);
     for (unsigned x = 0; x < iter->second.size(); x++) {
-      _item = radio_item(item, highlighters_radio, manager.get_language(iter->second[x])->get_name());
-      _item->signal_activate().connect(sigc::bind<int>(sigc::mem_fun(*this, &MenuBar::signal_highlighter_activate_cb), iter->second[x]));
-      highlighters.push_back(_item);
-      manager.set_map(iter->second[x], highlighters.size()-1);
+      _item = radio_item(item, highlighters_radio, SourceManager::get_name(iter->second[x]));
+      _item->property_user_data().set_value(const_cast<char *>(iter->second[x].c_str()));
+      _item->signal_activate().connect(sigc::bind<std::string>(sigc::mem_fun(*this, &MenuBar::signal_highlighter_activate_cb), iter->second[x]));
     }
   }
 }
 
-void MenuBar::signal_highlighter_activate_cb(int x) {
+void MenuBar::signal_highlighter_activate_cb(std::string id) {
   if (_ignore_highlighting_changed_signal_hack == true) {
     return;
   }
 
-  if (dynamic_cast<Gtk::CheckMenuItem *>(highlighters[_conf.highlight_manager.get_map(x)])->get_active()) {
-    signal_highlighter_activate(x);
+  Gtk::Menu_Helpers::MenuList& items = highlight->items();
+  // Iterate over the items.
+  for (Gtk::Menu_Helpers::MenuList::iterator iter = items.begin(); iter != items.end(); iter++) {
+    Gtk::Menu *m = iter->get_submenu();
+    if (m) {
+      Gtk::Menu_Helpers::MenuList& items2 = m->items();
+      // Iterate over the items.
+      for (Gtk::Menu_Helpers::MenuList::iterator iter2 = items2.begin(); iter != items2.end(); iter++) {
+	if (id == static_cast<char *>(iter2->property_user_data().get_value())) {
+	  Gtk::CheckMenuItem *it = dynamic_cast<Gtk::CheckMenuItem *>(&(*iter2));
+	  if (it && it->get_active()) {
+	    signal_highlighter_activate.emit(id);
+	    return;
+	  }
+	}
+      }
+    }
+    else {
+      // Probably this is the None item.
+      Gtk::CheckMenuItem *it = dynamic_cast<Gtk::CheckMenuItem *>(&(*iter));
+      if (it && it->get_active()) {
+	signal_highlighter_activate.emit("");
+	return;
+      }
+    }
   }
 }
 
-void MenuBar::set_highlight(int x) {
+void MenuBar::set_highlight(std::string id) {
   _ignore_highlighting_changed_signal_hack = true;
-  dynamic_cast<Gtk::CheckMenuItem *>(x == -1 ? highlighters[0] : highlighters[_conf.highlight_manager.get_map(x)])->set_active(true);
+
+  Gtk::Menu_Helpers::MenuList& items = highlight->items();
+  if (id == "") {
+    Gtk::CheckMenuItem *item = dynamic_cast<Gtk::CheckMenuItem *>(&(*items.begin()));
+    if (item) {
+      item->set_active(true);
+      _ignore_highlighting_changed_signal_hack = false;
+      return;
+    }
+  }
+  else {
+    // Iterate over the items.
+    for (Gtk::Menu_Helpers::MenuList::iterator iter = items.begin(); iter != items.end(); iter++) {
+      // We know they are all Gtk::Menu objects
+      Gtk::Menu *m = iter->get_submenu();
+      if (!m) {
+	continue;
+      }
+      Gtk::Menu_Helpers::MenuList& items2 = m->items();
+
+      for (Gtk::Menu_Helpers::MenuList::iterator iter2 = items2.begin(); iter2 != items2.end(); iter2++) {
+	if (id == static_cast<char *>(iter2->property_user_data().get_value())) {
+	  Gtk::CheckMenuItem *it = dynamic_cast<Gtk::CheckMenuItem *>(&(*iter2));
+	  if (it) {
+	    it->set_active(true);
+	    _ignore_highlighting_changed_signal_hack = false;
+	    return;
+	  }
+	}
+      }
+    }
+  }
+  // Just in case.
   _ignore_highlighting_changed_signal_hack = false;
 }
 #endif
