@@ -27,9 +27,7 @@
 #include "openlocationdialog.hh"
 #include <gtkmm/stock.h>
 #include "macros.h"
-#include "pipe.hh"
 #include "utils.hh"
-#include "thread.hh"
 
 OpenLocationDialog::OpenLocationDialog(Conf& conf, Encodings& enc) : _conf(conf) {
   set_modal();
@@ -67,8 +65,9 @@ OpenLocationDialog::OpenLocationDialog(Conf& conf, Encodings& enc) : _conf(conf)
   box1.set_border_width(5);
   box2.set_border_width(5);
 
-  add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
+
   add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+  add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_ACCEPT);
 
   // TODO: Any idea why do I need to set this explicitly ?
   location.set_size_request(500, -1);
@@ -78,44 +77,27 @@ OpenLocationDialog::~OpenLocationDialog() {
 
 }
 
+std::string& OpenLocationDialog::get_uri() {
+  return uri;
+}
+
 bool OpenLocationDialog::run(std::string& out) {
   show_all();
 
   if (Gtk::Dialog::run() == Gtk::RESPONSE_ACCEPT) {
     hide();
     Gtk::Entry *entry = location.get_entry();
-    std::string loc = entry->get_text();
-    if (loc.size() == 0) {
+    uri = entry->get_text();
+    if (uri.size() == 0) {
       out = _("You must enter a location to open.");
       return false;
     }
-
-    Dispatcher *dispatcher = new Dispatcher;
-    dispatcher->in = loc;
-
-    sigc::slot<void> slot = sigc::bind<Dispatcher *>(sigc::mem_fun(*this, &OpenLocationDialog::get), dispatcher);
-    Thread thread(slot, dispatcher);
-
-    std::string error;
-    if (thread.ok(error)) {
-      if (!dispatcher->err) {
-	_conf.locations_prepend(loc);
-	out = dispatcher->out;
-	delete dispatcher;
-	return true;
-      }
-      else {
-	out = dispatcher->error;
-	delete dispatcher;
-	return false;
-      }
-    }
     else {
-      out = error;
-      delete dispatcher;
-      return false;
+      _conf.locations_prepend(uri);
+      return true;
     }
   }
+
   return false;
 }
 
@@ -123,17 +105,6 @@ int OpenLocationDialog::get_encoding() {
   return _encodings.get(encoding.get_active_text());
 }
 
-void OpenLocationDialog::get(Dispatcher *dispatcher) {
-  // Let's get it.
-  std::string command(_conf.get("remote_command", REMOTE_COMMAND));
-  if (command.size() == 0) {
-    command = REMOTE_COMMAND;
-  }
-
-  dispatcher->err = !Pipe::exec(command, Utils::substitute(command, dispatcher->in), dispatcher->out, dispatcher->error);
-  dispatcher->emit();
-}
-
 bool OpenLocationDialog::insert_to_active() {
   return to_active.get_active();
 }
