@@ -41,7 +41,9 @@ Statusbar::Statusbar(Conf& conf) :
 
   pack_start(enc, false, false);
   pack_start(overwrite, false, false);
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
   pack_start(input, false, false);
+#endif
   pack_start(sbar, false, false);
 
   enc.set_size_request(150, -1);
@@ -50,12 +52,11 @@ Statusbar::Statusbar(Conf& conf) :
 
   set_overwrite(false);
   set_position(1, 1);
-
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
   input.set_label(_("Input"));
   input.signal_toggled().connect(sigc::mem_fun(*this, &Statusbar::signal_input_toggled_cb));
-
   activate_input(false);
-
+#endif
   show_all();
   red.hide();
 }
@@ -97,6 +98,7 @@ void Statusbar::set_encoding(std::string e) {
   enc.set_text(e);
 }
 
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
 bool Statusbar::set_input_status(bool active) {
   if (active == input.get_active()) {
     return false;
@@ -118,3 +120,4 @@ void Statusbar::signal_input_toggled_cb() {
 void Statusbar::activate_input(bool active) {
   input.set_sensitive(active);
 }
+#endif
