@@ -26,10 +26,16 @@
 
 #include "textbuffer.hh"
 #include "utils.hh"
+#ifdef ENABLE_EMULATOR
+#include "emulator.hh"
+#endif
+#ifdef ENABLE_MULTIPRESS
+#include "multipress.hh"
+#endif
 
-TextBuffer::TextBuffer()
+TextBuffer::TextBuffer(Conf& conf) : _conf(conf)
 #ifdef ENABLE_HIGHLIGHT
-  :  Gtk::TextBuffer(GTK_TEXT_BUFFER(gtk_source_buffer_new(NULL)))
+    , Gtk::TextBuffer(GTK_TEXT_BUFFER(gtk_source_buffer_new(NULL)))
 #endif
 {
   _insert = create_mark(begin());
@@ -38,6 +44,10 @@ TextBuffer::TextBuffer()
   // We don't want gtksourceview to manage the undo for us. We are doing it anyway.
   gtk_source_buffer_set_max_undo_levels(GTK_SOURCE_BUFFER(gobj()), 0);
 #endif
+#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+  on_insert_conn = signal_insert().connect(sigc::mem_fun(this, &TextBuffer::on_insert), false);
+  signal_erase().connect(sigc::mem_fun(this, &TextBuffer::on_erase), false);
+#endif
 }
 
 TextBuffer::~TextBuffer() {
@@ -45,7 +55,7 @@ TextBuffer::~TextBuffer() {
   //  delete_mark(_insert);
 }
 
-void TextBuffer::on_insert (const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes) {
+void TextBuffer::on_insert(const Gtk::TextBuffer::iterator& pos, const Glib::ustring& text, int bytes) {
   move_mark(_insert, pos);
   Glib::ustring utf8;
   // Here we will handle the lam-alef inserts.
@@ -66,25 +76,65 @@ void TextBuffer::on_insert (const Gtk::TextBuffer::iterator& pos, const Glib::us
   // Finally: THIS SUCK A LOT.
   // Someone should contact the xorg guys and see.
   if (Utils::is_lam_alef(text, utf8)) {
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
     Gtk::TextBuffer::on_insert(pos, utf8, utf8.bytes());
+#else
+    on_insert_conn.block();
+    insert(pos, utf8);
+    on_insert_conn.unblock();
+    g_signal_stop_emission_by_name(gobj(), "insert-text");
+#endif
   }
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
+  else if (text.size() == 1) {
+    std::string val;
+    if (Emulator::get_active()) {
+      if (Emulator::get(text, val)) {
+	// insert.
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+	Gtk::TextBuffer::on_insert(pos, val, val.size());
+#else
+	on_insert_conn.block();
+	insert(pos, val);
+	on_insert_conn.unblock();
+	g_signal_stop_emission_by_name(gobj(), "insert-text");
+#endif
+      }
+    }
+    else if (Multipress::get_active()) {
+      if (Multipress::get(text, _conf.get("multipress_timeout", 1000))) {
+	// We do nothing here. Multipress will call us later.
+	g_signal_stop_emission_by_name(gobj(), "insert-text");
+      }
+    }
+    else {
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
+      Gtk::TextBuffer::on_insert(pos, text, bytes);
+#endif
+    }
+  }
+#endif
   else {
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
     Gtk::TextBuffer::on_insert(pos, text, bytes);
+#endif
   }
 }
 
 void TextBuffer::on_erase(const Gtk::TextBuffer::iterator& start, const Gtk::TextBuffer::iterator& end) {
   _erase = end.get_line();
   _deleted = get_text(start, end, false);
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   Gtk::TextBuffer::on_erase(start, end);
+#endif
 }
 
 #ifdef ENABLE_HIGHLIGHT
 void TextBuffer::set_highlight(bool s) {
-  gtk_source_buffer_set_highlight(GTK_SOURCE_BUFFER(gobj()), s ?  TRUE : FALSE);
+  gtk_source_buffer_set_highlight_syntax(GTK_SOURCE_BUFFER(gobj()), s ?  TRUE : FALSE);
 }
 
-void TextBuffer::set_language(SourceLanguage*& lang) {
-  gtk_source_buffer_set_language(GTK_SOURCE_BUFFER(gobj()), lang->gobj());
+void TextBuffer::set_language(GtkSourceLanguage* lang) {
+  gtk_source_buffer_set_language(GTK_SOURCE_BUFFER(gobj()), lang);
 }
 #endif
