@@ -27,13 +27,9 @@
 #include "toolbar.hh"
 #include <gtkmm/stock.h>
 #include "macros.h"
-#ifdef HAVE_SPELL
-#include "spell.hh"
-#endif
 #include "utils.hh"
 #include <fstream>
 #include <iostream>
-#include "tempfile.hh"
 
 Toolbar::Toolbar(Conf& conf) :
   _conf(conf),
@@ -50,9 +46,12 @@ Toolbar::Toolbar(Conf& conf) :
   _copy(Gtk::Stock::COPY),
   _paste(Gtk::Stock::PASTE),
   _erase(Gtk::Stock::DELETE),
+#ifdef ENABLE_MAEMO
+  _full_screen(Gtk::Stock::ZOOM_FIT),
+#endif
   _go_to_l(_("Goto Line")),
   _search_l(_("Search"))
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   ,
   _dictionary_l(_("Spelling Dictionary"))
 #endif
@@ -90,6 +89,9 @@ void Toolbar::create_main() {
   _main.append(_copy);
   _main.append(_paste);
   _main.append(_erase);
+#ifdef ENABLE_MAEMO
+  _main.append(_full_screen);
+#endif
 
   // Our tooltips.
   // NOTE:
@@ -110,6 +112,9 @@ void Toolbar::create_main() {
   _copy.set_tooltip(*tips, _("Copy"));
   _paste.set_tooltip(*tips, _("Paste"));
   _erase.set_tooltip(*tips, _("Delete current selection"));
+#ifdef ENABLE_MAEMO
+  _full_screen.set_tooltip(*tips, _("Toggle full screen mode"));
+#endif
 
   // This is for both_horiz style
   // NOTE: http://mail.gnome.org/archives/gtkmm-list/2004-June/msg00112.html
@@ -126,6 +131,9 @@ void Toolbar::create_main() {
   _copy.set_is_important();
   _paste.set_is_important();
   _erase.set_is_important();
+#ifdef ENABLE_MAEMO
+  _full_screen.set_is_important();
+#endif
 
   _main.show_all();
 
@@ -143,6 +151,9 @@ void Toolbar::create_main() {
   _copy.signal_clicked().connect(sigc::mem_fun(signal_copy_clicked, &sigc::signal<void>::emit));
   _paste.signal_clicked().connect(sigc::mem_fun(signal_paste_clicked, &sigc::signal<void>::emit));
   _erase.signal_clicked().connect(sigc::mem_fun(signal_erase_clicked, &sigc::signal<void>::emit));
+#ifdef ENABLE_MAEMO
+  _full_screen.signal_clicked().connect(sigc::mem_fun(signal_full_screen_clicked, &sigc::signal<void>::emit));
+#endif
 }
 
 void Toolbar::create_extended() {
@@ -152,7 +163,7 @@ void Toolbar::create_extended() {
   box.pack_start(_go_to_l, false, false, 10);
   box.pack_start(_go_to, false, false);
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   box.pack_start(_dictionary_l, false, false, 10);
   box.pack_start(_dictionary, false, false);
 
@@ -162,11 +173,6 @@ void Toolbar::create_extended() {
   _spell.set_relief(Gtk::RELIEF_NONE);
   _spell.signal_clicked().connect(sigc::mem_fun(signal_spell_clicked, &sigc::signal<void>::emit));
 
-  std::vector<std::string> dicts;
-  katoob_spell_list_available(dicts);
-  for (unsigned x = 0; x < dicts.size(); x++) {
-    _dictionary.append_text(dicts[x]);
-  }
   _dictionary.set_active_text(_conf.get("default_dict", "en"));
   // TODO: I want to connect without the callback step.
   signal_dictionary_changed_conn = _dictionary.signal_changed().connect(sigc::mem_fun(*this, &Toolbar::dictionary_changed_cb));
@@ -204,7 +210,7 @@ void Toolbar::set_beside() {
   _conf.set("toolbartype","both_horiz");
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void Toolbar::set_dictionary(std::string& d) {
   signal_dictionary_changed_conn.block();
   _dictionary.set_active_text(d);
@@ -221,7 +227,11 @@ void Toolbar::reset_gui() {
   _conf.get("extended_toolbar", true) == true ? _extended.show() : _extended.hide();
   _conf.get("extra_buttons", true) == true ? _extra_buttons.show_all() : _extra_buttons.hide();
 
+#ifdef ENABLE_MAEMO
+  const std::string& toolbartype = _conf.get("toolbartype", "icons");
+#else
   const std::string& toolbartype = _conf.get("toolbartype", "both");
+#endif
   if (toolbartype == "text") {
     set_text();
   }
@@ -248,15 +258,13 @@ void Toolbar::reset_gui(bool enable) {
   _copy.set_sensitive(enable);
   _paste.set_sensitive(enable);
   _erase.set_sensitive(enable);
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   _dictionary.set_sensitive(enable);
   _spell.set_sensitive(enable);
   _dictionary.set_sensitive(enable);
 #endif
   _go_to.set_sensitive(enable);
-#ifdef HAVE_SPELL
   _search.set_sensitive(enable);
-#endif
   _extra_buttons.set_sensitive(enable);
 }
 
@@ -278,58 +286,19 @@ void Toolbar::create_extra_buttons() {
   ifs.open(path.c_str());
   if (ifs.is_open()) {
     std::string buff;
-    while (getline(ifs, buff))
-      if (!create_extra_button(buff))
-	break;
+    while (getline(ifs, buff)) {
+      create_extra_button(buff);
+    }
     ifs.close();
   }
 }
 
-bool Toolbar::create_extra_button(std::string& label) {
-  std::string error;
-  TempFile tf;
-  if (!tf.ok(error)) {
-#if ((defined(_WIN32) && !defined(NDEBUG)) || (!defined(_WIN32)))
-    std::cout << "Error creating temp file for icon. " << error << std::endl;
-#endif
-    return false;
-    }
-
-  // TODO: We will do it the C way until we get G++ bindings for cairo and pangocairo.
-  cairo_surface_t *s = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 20, 20);
-  cairo_t *t = cairo_create(s);
-  PangoLayout *layout = pango_cairo_create_layout(t);
-  PangoFontDescription *desc = pango_font_description_from_string("Sans 20");
-  pango_layout_set_font_description(layout, desc);
-  pango_font_description_free(desc);
-
-  int w, h;
-  // Let's clear it.
-  cairo_set_source_rgb(t, 1,1,1);
-  cairo_paint(t);
-  pango_layout_set_text(layout, label.c_str(), -1);
-  cairo_set_source_rgb(t, 0, 0, 0);
-  pango_cairo_update_layout(t, layout);
-  pango_layout_get_size (layout, &w, &h);
-  pango_cairo_show_layout(t, layout);
-  if (cairo_surface_write_to_png(s, tf.get_name().c_str()) != CAIRO_STATUS_SUCCESS) {
-    cairo_destroy (t);
-    cairo_surface_destroy(s);
-    g_object_unref (layout);
-    return false;
-  }
-  // Now let's create our button.
+void Toolbar::create_extra_button(std::string& label) {
   Gtk::ToolButton *button = Gtk::manage(new Gtk::ToolButton);
-
-  Gtk::Image *image = Gtk::manage(new Gtk::Image(tf.get_name()));
-  button->set_icon_widget(*image);
+  button->set_label(label);
   button->show_all();
   _extra_buttons.append(*button);
-  cairo_destroy(t);
-  cairo_surface_destroy(s);
-  g_object_unref(layout);
   button->signal_clicked().connect(sigc::bind<std::string>(sigc::mem_fun(*this, &Toolbar::extra_button_clicked), label));
-  return true;
 }
 
 void Toolbar::enable_undo(bool s) {
@@ -340,7 +309,7 @@ void Toolbar::enable_redo(bool s) {
   _redo.set_sensitive(s);
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void Toolbar::enable_dictionary(bool s) {
   _dictionary.set_sensitive(s);
 }
