@@ -29,19 +29,24 @@
 KatoobDoElem::KatoobDoElem(KatoobDoType action, const std::string& t, int p) :
   _text(t),
   _pos(p),
-  _action(action)
-{
+  _action(action) {
 }
 
-KatoobDoElem::~KatoobDoElem()
-{
-
+KatoobDoElem::~KatoobDoElem() {
 }
 
-void KatoobDoElem::toggle()
-{
-  if (_action == KATOOB_DO_INSERT)
-    _action = KATOOB_DO_DELETE;
-  else
-    _action = KATOOB_DO_INSERT;
+void KatoobDoElem::toggle() {
+  _action = _action == KATOOB_DO_INSERT ? KATOOB_DO_DELETE : KATOOB_DO_INSERT;
+}
+
+std::string& KatoobDoElem::text() {
+  return _text;
+}
+
+int KatoobDoElem::pos() {
+  return _pos;
+}
+
+KatoobDoType& KatoobDoElem::action() {
+  return _action;
 }
