@@ -33,6 +33,14 @@
 #include <gtkmm/textview.h>
 #include "macros.h"
 
+#ifdef _WIN32
+#define __KATOOB_CONF_DIR__ "Katoob"
+#elif defined(ENABLE_MAEMO)
+#define __KATOOB_CONF_DIR__ ".osso/katoob"
+#else /* ! _WIN32 */
+#define __KATOOB_CONF_DIR__ ".katoob"
+#endif
+
 std::string Utils::get_data_dir() {
 #ifndef _WIN32
   return std::string(PREFIX + Utils::get_dir_separator() + "share" + Utils::get_dir_separator() + PACKAGE + Utils::get_dir_separator());
@@ -42,7 +50,7 @@ std::string Utils::get_data_dir() {
 #endif
 }
 
-std::string Utils::get_data_path(char *str) {
+std::string Utils::get_data_path(const char *str) {
   std::string s(get_data_dir() + get_dir_separator() + str);
   return s;
 }
@@ -143,7 +151,6 @@ void Utils::katoob_set_color(Conf& cf, Gtk::Label *l, KatoobColor c) {
 }
 
 bool Utils::katoob_write(Conf& conf, std::string& file, std::string& text, std::string& error) {
-  // TODO: I'll do it the gtk style until glibmm wraps file_set_contents().
   gchar *f = NULL;
   GError *er = NULL;
 
@@ -234,9 +241,9 @@ bool Utils::katoob_write(const char *file, const char *text, unsigned len, std::
   return true;
 }
 
-bool Utils::katoob_read(std::string& file, std::string& out) {
+bool Utils::katoob_read(const std::string& file, std::string& out) {
   if (Glib::file_test(file, Glib::FILE_TEST_IS_DIR)) {
-    out = file + _(" Is a directory.");
+    out = substitute(_("%s is a directory."), file);
     return false;
   }
 
@@ -331,6 +338,20 @@ std::string Utils::substitute(const std::string& orig, const int a, const int b,
   return final_str;
 }
 
+std::string Utils::substitute(const std::string& orig, const std::string& a, const int b) {
+  gchar *tmp_str = g_strdup_printf(orig.c_str(), a.c_str(), b);
+  std::string final_str(tmp_str);
+  g_free(tmp_str);
+  return final_str;
+}
+
+std::string Utils::substitute(const std::string& orig, const std::string& a, const int b, const std::string& c, const std::string& d) {
+  gchar *tmp_str = g_strdup_printf(orig.c_str(), a.c_str(), b, c.c_str(), d.c_str());
+  std::string final_str(tmp_str);
+  g_free(tmp_str);
+  return final_str;
+}
+
 std::string Utils::katoob_get_default_font() {
   Gtk::TextView tv;
   tv.show();
@@ -359,3 +380,128 @@ std::vector<std::string> Utils::split(const std::string& src, const char n) {
   }
   return result;
 }
+
+bool Utils::lock_file(int fd, std::string& error) {
+  struct flock lock;
+  lock.l_type = F_WRLCK;
+  lock.l_whence = SEEK_SET;
+  lock.l_start = 0;
+  lock.l_len = 0;
+
+  if (fcntl(fd, F_SETLK, &lock) == -1) {
+    error = std::strerror(errno);
+    return false;
+  }
+  return true;
+}
+
+bool Utils::unlock_file(int fd, std::string& error) {
+  struct flock lock;
+  lock.l_type = F_UNLCK;
+  lock.l_whence = SEEK_SET;
+  lock.l_start = 0;
+  lock.l_len = 0;
+
+  if (fcntl(fd, F_SETLK, &lock) == -1) {
+    error = std::strerror(errno);
+    return false;
+  }
+  return true;
+}
+
+bool Utils::file_is_locked(const std::string& file) {
+  int fd = ::open(file.c_str(), O_RDONLY);
+  if (fd == -1) {
+    std::cerr << "Failed to open " << file.c_str() << " " << std::strerror(errno) << std::endl;
+    return true;
+  }
+  bool locked = file_is_locked(fd);
+
+  if (::close(fd) == -1) {
+    std::cerr << "Failed to close " << file.c_str() << " " << std::strerror(errno) << std::endl;
+    return true;
+  }
+  return locked;
+}
+
+bool Utils::file_is_locked(int fd) {
+  struct flock lock;
+  lock.l_type = F_WRLCK;
+  lock.l_whence = SEEK_SET;
+  lock.l_start = 0;
+  lock.l_len = 0;
+  lock.l_pid = 0;
+
+  if (fcntl(fd, F_GETLK, &lock) == -1) {
+    std::cerr << "Failed to check for locking: " << std::strerror(errno) << std::endl;
+    return false;
+  }
+  return lock.l_pid != 0;
+}
+
+std::string Utils::get_conf_dir() {
+  static std::string _dir;
+  if (_dir.size() == 0) {
+    _dir = Glib::build_filename(Glib::get_home_dir(), __KATOOB_CONF_DIR__);
+  }
+  return _dir;
+}
+
+std::string Utils::get_recovery_dir() {
+  static std::string _dir;
+  if (_dir.size() == 0) {
+    _dir = Glib::build_filename(Utils::get_conf_dir(), "recover");
+  }
+  return _dir;
+}
+
+bool Utils::create_recovery_file(std::string& file, int& fd) {
+  file = Glib::build_filename(get_recovery_dir(), get_recovery_template());
+  fd = Glib::mkstemp(file);
+  return fd != -1;
+}
+
+std::string Utils::get_recovery_template(std::string suffix) {
+  return "katoob_autosave_"+suffix;
+}
+
+bool Utils::get_recovery_files(std::map<std::string, std::string>& files, std::string& error) {
+  try {
+    Glib::Dir dir(get_recovery_dir());
+    std::vector<std::string> entries(dir.begin(), dir.end());
+    dir.close();
+    for (unsigned x = 0; x < entries.size(); x++) {
+      if (entries[x].find(get_recovery_template("")) == std::string::npos) {
+	std::cerr << "Skipping " << entries[x] << std::endl;
+	continue;
+      }
+
+      std::string file = Glib::build_filename(get_recovery_dir(), entries[x]);
+      // is it free ?
+      if (file_is_locked(file)) {
+	std::cerr << "Skipping locked file " << file << std::endl;
+	continue;
+      }
+
+      // Try to read it:
+      try {
+	std::string str = Glib::file_get_contents(file);
+	if (str.size() == 0) {
+	  std::cerr << "Erasing zero sized file " << file << std::endl;
+	  unlink(file.c_str());
+	  continue;
+	}
+	files[file] = str;
+      }
+      catch (Glib::FileError& err) {
+	std::cerr << "Failed to read " << file << ": " << err.what() << std::endl;
+	continue;
+      }
+    }
+    return true;
+  }
+  catch (Glib::FileError& err) {
+    error = err.what();
+    return false;
+  }
+}
