@@ -37,14 +37,29 @@
 Window::Window(Conf& conf, Encodings& encodings, std::vector<std::string>& files) :
   _conf(conf),
   _encodings(encodings),
-  menubar(conf, encodings, _emulator.list_layouts(), _multipress.list_layouts()),
+  menubar(conf, encodings
+#ifdef ENABLE_EMULATOR
+	  , Emulator::list_layouts()
+#endif
+#ifdef ENABLE_MULTIPRESS
+	  , Multipress::list_layouts()
+#endif
+	  ),
   toolbar(conf),
   mdi(conf, encodings),
   statusbar(conf),
   box(false, 0) {
+#ifdef ENABLE_MAEMO
+  Hildon::Window::set_menu(menubar);
+  Hildon::Window::add_toolbar(toolbar.get_main());
+  //  Hildon::Window::add_toolbar(toolbar.get_extended());
+#else
   box.pack_start(menubar, Gtk::PACK_SHRINK, 0);
   box.pack_start(toolbar.get_main(), false, false, 0);
+  //  box.pack_start(toolbar.get_extended(), Gtk::PACK_SHRINK, 0);
+#endif
   box.pack_start(toolbar.get_extended(), Gtk::PACK_SHRINK, 0);
+
   box.pack_start(mdi, Gtk::PACK_EXPAND_WIDGET, 0);
   box.pack_start(statusbar, Gtk::PACK_SHRINK, 0);
 
@@ -57,28 +72,33 @@ Window::Window(Conf& conf, Encodings& encodings, std::vector<std::string>& files
 
   reset_gui();
 
+  mdi.scan_temp();
   // This is here after we connect the signals
   // So we can get the signals emitted when we create the first document.
   // Now let's create the children.
-  if (files.size() == 0) {
+  // If MDI has an active Document then we managed to recover something.
+  // And in that case, we won't create a new document.
+  if ((files.size() == 0) && (!mdi.get_active())) {
     mdi.create_document();
   }
   else {
     for (unsigned int x = 0; x < files.size(); x++) {
       mdi.create_document(files[x]);
     }
-  }
-  files.clear();
 
-  // It's possible that we might fail to detect the encoding of all the files
-  // passed as arguments. In this case, we will create a blank document.
-  // We can't the number of the documents because MDI has no size method.
-  if (!mdi.get_active()) {
-    mdi.create_document();
+    files.clear();
+
+    // It's possible that we might fail to detect the encoding of all the files
+    // passed as arguments. In this case, we will create a blank document.
+    // We can't the number of the documents because MDI has no size method.
+    if (!mdi.get_active()) {
+      mdi.create_document();
+    }
   }
 
   move(conf.get("x", 50), conf.get("y", 50));
   resize (conf.get("w", 500), conf.get("h", 400));
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try {
     set_icon_from_file(Utils::get_data_path("katoob-small.png"));
   }
@@ -94,7 +114,13 @@ Window::Window(Conf& conf, Encodings& encodings, std::vector<std::string>& files
 #endif
  }
 #endif
-
+#else /* ! GLIBMM_EXCEPTIONS_ENABLED */
+ std::auto_ptr<Glib::Error> error;
+ set_icon_from_file(Utils::get_data_path("katoob-small.png"), error);
+ if (error.get()) {
+   std::cout << error->what() << std::endl;
+ }
+#endif
   //  set_title();
  signal_delete_event().connect(sigc::mem_fun(*this, &Window::signal_delete_event_cb));
 
@@ -107,17 +133,24 @@ Window::Window(Conf& conf, Encodings& encodings, std::vector<std::string>& files
   if (doc) {
     doc->grab_focus();
   }
-
+#if (defined ENABLE_EMULATOR) || (defined ENABLE_MULTIPRESS)
   std::string err;
-  if (!_emulator.ok(err)) {
+#endif
+#ifdef ENABLE_EMULATOR
+  if (!Emulator::ok(err)) {
     err += _(" The keyboard emulator will not work.");
     katoob_error(err);
   }
-
+#endif
+#ifdef ENABLE_MULTIPRESS
+  if (!Multipress::ok(err)) {
+    err += _(" Multipress will not work.");
+    katoob_error(err);
+  }
+#endif
   std::string ver = conf.get_version();
   if ((ver.size() == 0) && conf.ok()) {
-    std::string message(_("A lot of the configuration options changed in this version.\nPlease adjust the configuration first."));
-    katoob_info(message);
+    katoob_info(_("A lot of the configuration options have been changed in this version.\nPlease adjust the configuration first."));
     signal_preferences_activate_cb();
   }
 
@@ -136,14 +169,21 @@ Window::Window(Conf& conf, Encodings& encodings, std::vector<std::string>& files
   signal_drag_data_received().connect(sigc::mem_fun(*this, &Window::signal_drag_data_received_cb));
 
   // Multipress.
+#ifdef ENABLE_MULTIPRESS
   _multipress.signal_invalid_key.connect(sigc::mem_fun(*this, &Window::signal_invalid_key_cb));
   _multipress.signal_insert_key.connect(sigc::mem_fun(*this, &Window::signal_insert_key_cb));
-
+#endif
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
   // Statusbar input button.
   statusbar.signal_input_toggled.connect(mem_fun(*this, &Window::signal_input_toggled_cb));
   // The input window signals.
   input_window.signal_button_clicked.connect(sigc::mem_fun(*this, &Window::signal_insert_key_cb));
   input_window.signal_dialog_closed.connect(sigc::mem_fun(*this, &Window::signal_input_window_dialog_closed_cb));
+#endif
+#ifdef ENABLE_MAEMO
+  signal_window_state_event().connect(sigc::mem_fun(*this, &Window::signal_window_state_event_cb));
+  is_fullscreen = false;
+#endif
 }
 
 Window::~Window() {
@@ -164,8 +204,13 @@ void Window::connect_toolbar_signals() {
   toolbar.signal_copy_clicked.connect(sigc::mem_fun(mdi, &MDI::copy_cb));
   toolbar.signal_paste_clicked.connect(sigc::mem_fun(mdi, &MDI::paste_cb));
   toolbar.signal_erase_clicked.connect(sigc::mem_fun(mdi, &MDI::erase_cb));
+
+#ifdef ENABLE_MAEMO
+  toolbar.signal_full_screen_clicked.connect(sigc::mem_fun(*this, &Window::toggle_full_screen));
+#endif
+
   // The rest of the extended toolbar signals.
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   toolbar.signal_dictionary_changed.connect(sigc::mem_fun(*this, &Window::signal_dictionary_changed_cb));
   toolbar.signal_spell_clicked.connect(sigc::mem_fun(mdi, &MDI::do_spell));
 #endif
@@ -185,6 +230,8 @@ void Window::connect_menubar_signals() {
   menubar.signal_revert_activate.connect(sigc::mem_fun(mdi, &MDI::revert_cb));
 #ifdef ENABLE_PRINT
   menubar.signal_print_activate.connect(sigc::mem_fun(mdi, &MDI::print_cb));
+  menubar.signal_print_preview_activate.connect(sigc::mem_fun(mdi, &MDI::print_preview_cb));
+  menubar.signal_page_setup_activate.connect(sigc::mem_fun(mdi, &MDI::page_setup_cb));
 #endif
   menubar.signal_close_activate.connect(sigc::mem_fun(mdi, &MDI::close_cb));
   menubar.signal_quit_activate.connect(sigc::mem_fun(*this, &Window::signal_quit_activate_cb));
@@ -218,7 +265,7 @@ void Window::connect_menubar_signals() {
 
   menubar.signal_execute_activate.connect(sigc::mem_fun(mdi, &MDI::execute_cb));
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   menubar.signal_spell_activate.connect(sigc::mem_fun(mdi, &MDI::do_spell));
   signal_auto_spell_activate_conn = menubar.signal_auto_spell_activate.connect(sigc::mem_fun(mdi, &MDI::set_auto_spell));
 #endif
@@ -228,7 +275,9 @@ void Window::connect_menubar_signals() {
   menubar.signal_about_activate.connect(sigc::ptr_fun(&AboutDialog::run));
 
   menubar.signal_recent_activate.connect(sigc::mem_fun(mdi, &MDI::recent_cb));
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
   menubar.signal_layout_activate.connect(sigc::mem_fun(*this, &Window::signal_layout_activate_cb));
+#endif
   menubar.signal_encoding_activate.connect(sigc::mem_fun(*this, &Window::signal_encoding_activate_cb));
   menubar.signal_document_activate.connect(sigc::mem_fun(mdi, &MDI::activate));
 
@@ -244,7 +293,7 @@ void Window::connect_menubar_signals() {
 }
 
 void Window::connect_mdi_signals() {
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   mdi.signal_document_spell_enabled.connect(sigc::mem_fun(*this, &Window::on_document_spell_enabled_cb));
 #endif
   mdi.signal_document_wrap_text.connect(sigc::mem_fun(*this, &Window::signal_document_wrap_text_cb));
@@ -265,7 +314,7 @@ void Window::connect_mdi_signals() {
   mdi.signal_document_can_undo.connect(sigc::mem_fun(*this, &Window::signal_document_can_undo_cb));
   mdi.signal_document_modified.connect(sigc::mem_fun(*this, &Window::signal_document_modified_cb));
   mdi.signal_document_title_changed.connect(sigc::mem_fun(*this, &Window::signal_document_title_changed_cb));
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
   mdi.signal_document_dictionary_changed.connect(sigc::mem_fun(*this, &Window::signal_document_dictionary_changed_cb));
 #endif
 
@@ -282,7 +331,12 @@ void Window::connect_mdi_signals() {
 void Window::signal_drag_data_received_cb(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selection, guint info, guint time) {
   std::vector<std::string> files = selection.get_uris();
   std::string filename;
+#ifndef GLIBMM_EXCEPTIONS_ENABLED
+  std::auto_ptr<Glib::Error> error;
+#endif
+
   for (unsigned x = 0; x < files.size(); x++) {
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     try {
       filename = Glib::filename_from_uri(files[x]);
       mdi.create_document(filename);
@@ -290,16 +344,29 @@ void Window::signal_drag_data_received_cb(const Glib::RefPtr<Gdk::DragContext>&
     catch(Glib::ConvertError& e) {
       katoob_error(e.what());
     }
+#else
+    filename = Glib::filename_from_uri(files[x], error);
+    if (error.get()) {
+      katoob_error(error->what());
+      error.reset();
+    }
+#endif
   }
 }
 
 void Window::set_title(const char *str) {
+#ifdef ENABLE_MAEMO
+  if (str) {
+    Gtk::Window::set_title(str);
+  }
+#else
   std::stringstream title;
   if (str) {
     title << str << " - ";
   }
-  title << PACKAGE << " " << VERSION;
+  title << PACKAGE;
   Gtk::Window::set_title(title.str());
+#endif
 }
 
 void Window::signal_preferences_activate_cb() {
@@ -311,7 +378,7 @@ void Window::signal_preferences_activate_cb() {
   }
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void Window::on_document_spell_enabled_cb(bool s) {
   signal_auto_spell_activate_conn.block();
   menubar.enable_auto_spell(s);
@@ -336,79 +403,50 @@ void Window::signal_quit_activate_cb() {
   }
 }
 
-bool Window::on_key_press_event(GdkEventKey* event) {
-  // TODO: All our keyboard shortcuts are not working.
-
-  Document *doc = mdi.get_active();
-  if ((!doc) || (doc->get_readonly()) || (!doc->has_focus())) {
-    return Gtk::Window::on_key_press_event(event);
-  }
-
-  // We don't handle more than one key.
-  if (event->length != 1) {
-    return Gtk::Window::on_key_press_event(event);
-  }
-
-  gunichar x = gdk_keyval_to_unicode(event->keyval);
-  if (!x) {
-    return Gtk::Window::on_key_press_event(event);
-  }
-
-  if (_emulator.get_active()) {
-    Glib::ustring _key(1, x);
-    std::string key(_key);
-    std::string val;
-
-    if (_emulator.get(key, val)) {
-      doc->insert(val);
-      return true;
-    }
-    else {
-      return Gtk::Window::on_key_press_event(event);
-    }
-  }
-  else if (_multipress.get_active()) {
-    std::string val;
-    if (!_multipress.get(x, _conf.get("multipress_timeout", 1000))) {
-      // Something went wrong.
-      return Gtk::Window::on_key_press_event(event);
-    }
-    else {
-      // TODO: Is this right ?
-      return true;
-    }
-  }
-  else {
-    return Gtk::Window::on_key_press_event(event);
-  }
-}
-
+#if defined(ENABLE_MULTIPRESS) || defined(ENABLE_EMULATOR)
 void Window::signal_layout_activate_cb(int which, int x) {
   // TODO: Store the input_window state and restore it when we enable any.
   assert(which < 2);
+#ifdef ENABLE_EMULATOR
   if (which == 0) {
     _emulator.activate(x);
+#ifdef ENABLE_MULTIPRESS
     _multipress.activate(-1);
     statusbar.activate_input(true);
     if (input_window.is_visible()) {
       signal_input_toggled_cb(true);
     }
+#endif
   }
-  else if (which == 1) {
-    _multipress.activate(x);
-    _emulator.activate(-1);
-    statusbar.activate_input(true);
-    if (input_window.is_visible()) {
-      signal_input_toggled_cb(true);
+  else
+#endif
+#ifdef ENABLE_MULTIPRESS
+    if (which == 1) {
+
+      _multipress.activate(x);
+#ifdef ENABLE_EMULATOR
+      _emulator.activate(-1);
+#endif
+      statusbar.activate_input(true);
+      if (input_window.is_visible()) {
+	signal_input_toggled_cb(true);
+      }
     }
-  }
-  else {
-    _emulator.activate(x);
-    _multipress.activate(x);
-    statusbar.activate_input(false);
-    statusbar.set_input_status(false);
-  }
+    else
+#endif
+      {
+	// x is -1 here but I'll set it manually just in case.
+#ifdef ENABLE_EMULATOR
+	_emulator.activate(-1);
+#endif
+#ifdef ENABLE_MULTIPRESS
+	_multipress.activate(-1);
+	statusbar.activate_input(false);
+	statusbar.set_input_status(false);
+#endif
+      }
 }
+#endif
 
 void Window::signal_encoding_activate_cb(int e) {
   int x;
@@ -504,7 +542,7 @@ void Window::reset_gui() {
   */
 }
 
-#ifdef HAVE_SPELL
+#ifdef ENABLE_SPELL
 void Window::signal_dictionary_changed_cb(std::string d) {
   std::string old;
   if (!mdi.set_dictionary(old, d)) {
@@ -533,6 +571,7 @@ void Window::signal_line_numbers_activate_cb(bool s) {
   mdi.set_line_numbers(s);
 }
 
+#ifdef ENABLE_MULTIPRESS
 void Window::signal_insert_key_cb(std::string& str) {
   Document *doc = mdi.get_active();
   assert(doc != NULL);
@@ -543,24 +582,82 @@ void Window::signal_insert_key_cb(std::string& str) {
 }
 
 void Window::signal_invalid_key_cb(GdkEventKey *event) {
+#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
   Gtk::Window::on_key_press_event(event);
+#else
+  // TODO: This is not that good.
+  Document *doc = mdi.get_active();
+  if ((event->length > 0) && (doc) && (!doc->get_readonly())) {
+    gunichar ch = gdk_unicode_to_keyval(event->keyval);
+    if (ch != event->keyval | 0x01000000) {
+      gchar out[6];
+      int len = g_unichar_to_utf8(ch, out);
+      if (len != 0) {
+	std::string str(out, len);
+	doc->insert(str);
+      }
+    }
+  }
+#endif
 }
+#endif
 
+#if defined(ENABLE_EMULATOR) || defined(ENABLE_MULTIPRESS)
 void Window::signal_input_toggled_cb(bool active) {
   if (!active) {
     input_window.hide();
     return;
   }
 
+#ifdef ENABLE_EMULATOR
   if (_emulator.get_active()) {
     input_window.set_layout(_emulator.get_layout());
   }
-  else if (_multipress.get_active()) {
+  else
+#endif
+#ifdef ENABLE_MULTIPRESS
+    if (_multipress.get_active()) {
     input_window.set_layout(_multipress.get_layout());
   }
+#endif
   input_window.show();
 }
 
 void Window::signal_input_window_dialog_closed_cb() {
   statusbar.set_input_status(false);
 }
+#endif
+
+#ifdef ENABLE_MAEMO
+bool Window::signal_window_state_event_cb(GdkEventWindowState* event) {
+  if (event->type == GDK_WINDOW_STATE) {
+    if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN) {
+      if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN) {
+	is_fullscreen = true;
+      }
+      else {
+	is_fullscreen = false;
+      }
+    }
+  }
+
+  return false;
+}
+
+void Window::toggle_full_screen() {
+  if (is_fullscreen) {
+    unfullscreen();
+  }
+  else {
+    fullscreen();
+  }
+}
+
+void Window::signal_request_top_cb() {
+  present();
+}
+#endif
+
+void Window::autosave() {
+  mdi.autosave();
+}
